#include "Urdu Writer.h" 

HICON g_hIcon; // icon handle for both main and about dialog box.
LONG g_lpwpProcUrdu; // Original IDC_EB_URDU text box procedure.
HWND g_hwndUrdu; // Urdu text Box window handle (this text box is visible at run time).
HWND g_hwndPhonetic; // Phonetic text box window handle (this text box is not visible at run time).
HWND g_hwndDialog = NULL; // main dialog box window handle.
HWND g_hwndStartupScreen; // startup dialog box window handle (splash  screen).
CHAR g_lpCmdLine[MAX_PATH]; // buffer to save command line from WinMain(entry point function) for CmdLoadFile function.
BOOL g_bCmd; // flag value for checking if command line are existed (checked in WinMain entry point function and used in CmdLoadFile function).

CharMap g_cmChar[255]; // Phonetic(Urdu) keyboard mapping for character 0-255

// application entry point
int APIENTRY  WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	InitCommonControls();// themes and windows  XP  styles support stuff

	// Show splash screen at startup.
	g_hwndStartupScreen = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_STARTUP),NULL,(DLGPROC)StartupDlgProc);


	if (Check_Install_Font() == FALSE)// check if AlKatib1.TTF in fonts folder if not then add it.
	{
		// if font AlKatib1 is installed by us then close this application and rerun it.
		Sleep(4000); // 4 sec delay
		GetModuleFileName(NULL,g_lpCmdLine,MAX_PATH);// get this program complete path.

		// run this application again to properly recognize AlKatib1 font for displaying urdu.
		ShellExecute(NULL,"open",g_lpCmdLine,NULL,NULL,SW_SHOWNORMAL);
		EndDialog(g_hwndStartupScreen,NULL);// close splash screen
	}
	else// else stuff if Urdu font is already installed in system then run this application.
	{

		//---> Command Line Check Up
		// check if command line parameter are provided
		if (lstrlen(lpCmdLine) > 0	&& lstrlen(lpCmdLine) < MAX_PATH)
		{
			lstrcpy(g_lpCmdLine,lpCmdLine);// copy command line to buffer.
			g_bCmd = TRUE;// set flag true
		}
		else
		{
			g_bCmd = FALSE;// no command line set flag to false 
		}
		//---> ~Command Line Check Up

		// Load main icon for both main and about dialog box.
		g_hIcon=LoadIcon(hInstance, MAKEINTRESOURCE(IDI_0_MAIN) );

		Sleep(500);// half sec delay
		EndDialog(g_hwndStartupScreen,NULL);// close splash screen

		// create main dialog box
		DialogBox(hInstance, (LPCTSTR)IDD_MAIN, 0, (DLGPROC)DlgProc);

		// cleanup stuff
		DestroyIcon(g_hIcon);
	}

	// gentle go out
	return 0;
}

// main dialog box messages handler
LRESULT CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{

	case WM_INITDIALOG:// this message sent before displaying dialog
		g_hwndDialog = hwndDlg;// save this dialog box window handle (used in some other functions)

		SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_hIcon);// set icon //SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)LoadIcon(GetModuleHandle(NULL),MAKEINTRESOURCE(IDI_0_MAIN)));

		g_hwndUrdu = GetDlgItem(hwndDlg,IDC_EB_URDU);// save Urdu text box window handle for later use
		g_hwndPhonetic = GetDlgItem(hwndDlg,IDC_EB_PHONETIC);// save Phonetic text box window handle for later use

		// properly size Urdu text box in main dialog box
		Size2ParentClientArea(g_hwndUrdu,hwndDlg);

		// fill up phonetic character map for Urdu AlKatib1 font
		InitCharMap();

		// set text limit for Urdu text box
		SendMessage(GetDlgItem(hwndDlg,IDC_EB_URDU),EM_SETLIMITTEXT,TEXT_LIMIT - 5,0);
		// set text limit for Phonetic text box
		SendMessage(GetDlgItem(hwndDlg,IDC_EB_PHONETIC),EM_SETLIMITTEXT,TEXT_LIMIT - 5,0);
		// load and set AlKatib1 Urdu font to Urdu text box
		SendMessage(g_hwndUrdu,WM_SETFONT, (WPARAM) LoadFont_AlKatib1() , FALSE);

		// subclass Urdu text box so some needed keyboard message will sent to phonetic text box.
		g_lpwpProcUrdu= SetWindowLong(g_hwndUrdu,GWL_WNDPROC,(LONG)_UrduSubProc);// Subclass IDC_EB_URDU text box

		// command line file load up (if exist)
		CmdLoadFile();
		return true;
	case WM_SIZE:// maybe size is changing
		// properly size Urdu text box in main dialog box
		Size2ParentClientArea(g_hwndUrdu,hwndDlg);
		break;
	case WM_SHOWWINDOW:
		// check flag value if command line file (*.urp) is loaded then deselect all text.
		if (g_bCmd == 3)
		{
			SendMessage(g_hwndUrdu,EM_SETSEL,0,0);
			g_bCmd = TRUE;
		}
		break;

	case WM_COMMAND :
		switch( LOWORD( wParam ) )
		{
		case ID_MNU_OPEN:  // File -> Open menu item is clicked. 
			OpenUrduFile();// try to open file
			break;
		case ID_MNU_SAVE:// File -> Save As menu item is clicked. 
			SaveUrdu2File();// try to save phonetic text to file
			break;
		case ID_MNU_EXIT:// File -> Exit menu item is clicked. 
			EndDialog(hwndDlg,NULL);// close main dialog box (exit)
			break;
		case ID_MNU_COPY:// Edit ->Copy menu item is clicked.
			SendMessage(g_hwndUrdu,EM_SETSEL,0,-1);// select all text in Urdu text box
			SendMessage(g_hwndUrdu,WM_COPY,NULL,NULL);// copy selected text ( to clipboard ) from Urdu text box
			SendMessage(g_hwndUrdu,EM_SETSEL,0,0);// deselect text in Urdu text box
			break;
		case ID_MNU_CUT:// Edit ->Cut menu item is clicked.
			SendMessage(g_hwndUrdu,EM_SETSEL,0,-1);// select all text in Urdu text box
			SendMessage(g_hwndUrdu,WM_COPY,NULL,NULL);// copy selected text ( to clipboard ) from Urdu text box
			SetWindowText(g_hwndUrdu,"");// Clear Urdu text box
			SetWindowText(g_hwndPhonetic,"");// Clear Phonetic text box
			break;
		case ID_MNU_PASTE:// Edit ->Paste menu item is clicked.
			SendMessage(g_hwndUrdu,WM_PASTE,NULL,NULL);// paste text in Urdu text box from clipboard
			break;
		case ID_MNU_CLEAR: // Edit ->Clear menu item is clicked.
			SetWindowText(g_hwndUrdu,"");// Clear Urdu text box
			SetWindowText(g_hwndPhonetic,"");// Clear Phonetic text box
			break;
		case ID_MNU_ASSOCIATE:// Tools -> Associate with Urdu text file (*.urp) menu item is clicked
			SetFileAssociation();// Associate urp file with this application and notify system
			break;
		case ID_MNU_ABOUT:// Help -> About menu item clicked
			// show about dialog box
			DialogBox(GetModuleHandle(NULL), (LPCTSTR)IDD_DLG_ABOUT, hwndDlg, (DLGPROC)AboutDlgProc);
			break;
		}
		break;

	case WM_CLOSE:// maybe close button clicked on title bar		
		EndDialog(hwndDlg,0);// close main dialog (exit)
		break;
	case WM_DESTROY:
		// before destroying end up subclass of Urdu text box 
		SetWindowLong(g_hwndUrdu , GWL_WNDPROC, (LONG) g_lpwpProcUrdu );
		break;
	}
	return FALSE;
}

// About dialog box message handler
LRESULT CALLBACK AboutDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		// set icon
		SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_hIcon);
		return true;
	case WM_CLOSE:// maybe close button clicked on title bar
		EndDialog(hwndDlg,NULL);// close about dialog box
		break;
	}
	return FALSE;
}

// Splash screen (dialog box) messages handler
LRESULT CALLBACK StartupDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	if (uMsg == WM_INITDIALOG)
		return TRUE;

	return FALSE;
}

// Sub classed Urdu text box messages handler
LRESULT APIENTRY _UrduSubProc(HWND hwnd,UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRet;

	switch (uMsg)
	{
		// get some useful keyboard messages and send them to phonetic text box
	case WM_CHAR:
	case WM_KEYDOWN:
	case WM_KEYUP:

		// dont handle keyboard messages when Ctrl key is pressed 
		// for clipboard operations such as copy,cut,paste(Urdu text box).
		if (GetKeyState(VK_CONTROL ) < 0)
			break;

		lrRet =  SendMessage(g_hwndPhonetic,uMsg,wParam,lParam);// send keyboard related messages to Phonetic text box
		Phonetic2Urdu();// Convert Phonetic text into Urdu text
		return lrRet;
	}

	// any other messages goes to original Urdu text box procedure
	return CallWindowProc( (WNDPROC) g_lpwpProcUrdu, hwnd, uMsg, wParam, lParam); 
}

// Load AlKatib1 Urdu font and return handle for it.
HFONT LoadFont_AlKatib1(void)
{
	LOGFONT lf;

	ZeroMemory(&lf,sizeof(lf));

	lf.lfHeight = -28 ;
	lf.lfWeight = FW_NORMAL; 
	lf.lfCharSet = ANSI_CHARSET;
	lf.lfOutPrecision   = OUT_TT_PRECIS;
	lf.lfClipPrecision  = CLIP_DEFAULT_PRECIS;
	lf.lfQuality        = DEFAULT_QUALITY;
	lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;

	lstrcpy(lf.lfFaceName, "AlKatib1");

	return CreateFontIndirect(&lf);
}

// Save Phonetic text ( from phonetic text box ) to file.
void SaveUrdu2File(void)
{
	char lpszFile[MAX_PATH];
	char lpszText[TEXT_LIMIT];
	char lpszHeader[URP_HEADER_SIZE + 20];
	DWORD dwWrite;
	HANDLE hfNew;

	// get phonetic text from phonetic text box
	GetWindowText(g_hwndPhonetic,lpszText,TEXT_LIMIT);

	if (lstrlen(lpszText) == 0 ) // check if text exist in phonetic text box
	{
		MessageBox(g_hwndDialog,"Please type some text then try to save it.","Info",MB_ICONINFORMATION | MB_OK);
		return;
	}

	GetFileName_Save(lpszFile,g_hwndDialog);// show save dialog
	if (lstrlen(lpszFile) == 0 )// check if user enter file name
		return;

	// try to create a new file
	hfNew = CreateFile(lpszFile,
		GENERIC_WRITE,FILE_SHARE_READ,NULL,
		CREATE_NEW,FILE_ATTRIBUTE_NORMAL,
		NULL);

	if ( hfNew  == INVALID_HANDLE_VALUE )// check if new file is created 
	{
		MessageBox(g_hwndDialog,"File is not saved please try again with different file name or path.","Error",MB_ICONHAND | MB_OK);
		return;
	}

	// copy urp file header to buffer
	lstrcpy(lpszHeader,URP_HEADER);
	// write file header
	WriteFile(hfNew,lpszHeader,lstrlen(lpszHeader),&dwWrite,0);	
	// write phonetic text to file 
	WriteFile(hfNew,lpszText,lstrlen(lpszText),&dwWrite,0);
	// close new file
	CloseHandle(hfNew);
}

// check if command line exist (actually done WinMain entry point function).
// if command line exist then try to open file.
void CmdLoadFile(void)
{
	if (g_bCmd == TRUE)
	{
		// remove quote if existed.for example "e:\core\urdu.urp" to e:\core\urdu.urp
		PathUnquoteSpaces(g_lpCmdLine);
		// try to load phonetic(Urdu) text from file
		LoadUrduFromFile(g_lpCmdLine);
		// set flag value to 3 for deselect text in Urdu text box ( used in DlgProc-> case WM_SHOWWINDOW).
		g_bCmd = 3;
	}
}

// show open dialog box and try to load selected file (*.urp).
void OpenUrduFile(void)
{
	char lpszFile[MAX_PATH];

	GetFileName_Open(lpszFile,g_hwndDialog);// get file path
	if (lstrlen(lpszFile) == 0 )// check if user selected a file
		return;

	// try to load Urdu text file
	LoadUrduFromFile(lpszFile);
}
void LoadUrduFromFile(LPSTR lpUrduFile)
{
	HANDLE hfUrp;
	char lpszHeader[URP_HEADER_SIZE + 20];
	char lpszPhoneticText[TEXT_LIMIT  + 3];
	char lpMsgBuffer[256];
	char lpEndChar[1];
	DWORD dwRead;

	// try to open file
	hfUrp = CreateFile(lpUrduFile,
		GENERIC_READ,FILE_SHARE_READ,NULL,
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,
		NULL);

	if ( hfUrp==INVALID_HANDLE_VALUE )// check if file is opened
	{
		MessageBox(g_hwndDialog,"Cant open file.","Error",MB_ICONHAND | MB_OK);
		return;
	}

	// check for file size ( it must greater then urp file header size plus 1)
	if ( GetFileSize(hfUrp,NULL) < URP_HEADER_SIZE + 1)
	{
		MessageBox(g_hwndDialog,"This is not a Urdu text file.","Error",MB_ICONWARNING | MB_OK);
		return;
	}

	// read file header
	ReadFile(hfUrp,lpszHeader,URP_HEADER_SIZE,&dwRead,NULL);
	lpszHeader[dwRead] = 0;

	// check if this is a Urdu text file or not
	if ( strncmp(lpszHeader,URP_HEADER,URP_HEADER_SIZE - 3) != 0 )
	{
		MessageBox(g_hwndDialog,"This is not a Urdu text file.","Error",MB_ICONWARNING | MB_OK);
		return;
	}

	// check if this file require greater or other version of this application.
	if ( strcmp(lpszHeader,URP_HEADER) != 0 )
	{
		wsprintf(lpMsgBuffer,"This is a Urdu text file but it required Urdu (Pakistan) Text Writer version %s .", &lpszHeader[dwRead - 3]);
		MessageBox(g_hwndDialog,lpMsgBuffer,"Error",MB_ICONWARNING | MB_OK);	
		return;
	}

	// read all phonetic text from file
	ReadFile(hfUrp,lpszPhoneticText,TEXT_LIMIT,&dwRead,NULL);
	lpszPhoneticText[dwRead] = 0;

	// close the file
	CloseHandle(hfUrp);

	// check if we get some text
	if (lstrlen(lpszPhoneticText) > 0)
	{
		// add a space a end of phonetic text
		lstrcat(lpszPhoneticText," ");
		// set phonetic text to phonetic text box.
		SetWindowText(g_hwndPhonetic,lpszPhoneticText);

		//..........

		// select last two characters.
		SendMessage(g_hwndPhonetic,EM_SETSEL,GetWindowTextLength(g_hwndPhonetic)- 2,GetWindowTextLength(g_hwndPhonetic));

		// get last character (not space) from phonetic text.
		lpEndChar[0] = lpszPhoneticText[GetWindowTextLength(g_hwndPhonetic)- 2];
		lpEndChar[1] = 0;

		// replace selected text in phonetic text box 
		SendMessage(g_hwndPhonetic,EM_REPLACESEL,FALSE,(LPARAM)lpEndChar);

		//..........

		// Convert Phonetic text into Urdu text
		Phonetic2Urdu();
	}
}

// show save dialog box and get file name.
void GetFileName_Save(LPSTR lpszSaveFile ,HWND hWndParent)
{
	OPENFILENAME ofn;
	char szFileName[MAX_PATH]; 

	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hWndParent;
	ofn.lpstrFile = szFileName;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFileName);
	ofn.lpstrFilter = "Urdu(Pakistan)Text Writer Files (*.urp)\0*.urp\0\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrDefExt="urp";// default file extension (*.urp)
	ofn.Flags = OFN_HIDEREADONLY;// ???

	if (GetSaveFileName(&ofn)==TRUE)
		lstrcpy(lpszSaveFile,szFileName);// copy file name
	else
		lpszSaveFile[0] = 0 ;// nothing selected return null.
}

// show open dialog box and get file name.
void GetFileName_Open(LPSTR lpszOpenFile,HWND hWndParent)
{
	OPENFILENAME ofn;
	char szFileName[MAX_PATH]; 

	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hWndParent;
	ofn.lpstrFile = szFileName;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFileName);
	ofn.lpstrFilter = "Urdu(Pakistan)Text Writer Files (*.urp)\0*.urp\0\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if (GetOpenFileName(&ofn)==TRUE)
		lstrcpy(lpszOpenFile,szFileName);// copy file name
	else
		lpszOpenFile[0] = 0 ;// nothing selected return null.
}

// Set *.urp (Urdu Text File) Association with this application.
void SetFileAssociation(void)
{
	HKEY keyHandle;
	DWORD disposition;
	char lpszExePath[MAX_PATH];
	char lpszPathBuffer[MAX_PATH + 20];

	// get this executable file name and full path for example e:\urdu\Urdu(Pakistan) Text Writer.exe
	GetModuleFileName(NULL,lpszExePath,MAX_PATH);
	//.......>

	lpszPathBuffer[0] = 0;// null out string buffer
	// copy urp file description to buffer
	lstrcpy(lpszPathBuffer,"Urdu text file created with Urdu(Pakistan) text writer by Muhammad Arshad Latti.");

	// create "urp_v1.0" shell interface.
	RegCreateKeyEx(HKEY_CLASSES_ROOT,
		"urp_v1.0",0, 
		NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,
		NULL, &keyHandle,&disposition);

	// set urp file description string.
	RegSetValueEx( keyHandle, NULL, 0, REG_SZ,
		(LPBYTE)lpszPathBuffer, lstrlen(lpszPathBuffer) + 1 );

	// close registry key
	RegCloseKey(keyHandle);
	//.......>

	// append executable file string and add command line string in it 
	// for example c:\Urdu Pakistan\text writer.exe to "c:\Urdu Pakistan\text writer.exe" "%1"
	lpszPathBuffer[0] = 0;
	lstrcat(lpszPathBuffer,"\"");
	lstrcat(lpszPathBuffer,lpszExePath);
	lstrcat(lpszPathBuffer,"\" \"%1\"");

	// create open default verb for "urp_v1.0" shell interface.
	RegCreateKeyEx(HKEY_CLASSES_ROOT,
		"urp_v1.0\\shell\\Open\\command",0, 
		NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,
		NULL, &keyHandle,&disposition);

	// use this application to open the urp file.
	RegSetValueEx( keyHandle, NULL, 0, REG_SZ,
		(LPBYTE)lpszPathBuffer, lstrlen(lpszPathBuffer) + 1 );

	// close registry key
	RegCloseKey(keyHandle);
	//.......>

	// append executable file string for icon path for example c:\prog.exe to c:\prog.exe,1
	lpszPathBuffer[0] = 0;
	lstrcat(lpszPathBuffer,lpszExePath);
	lstrcat(lpszPathBuffer,",1");

	// create the default icon for urp_v1.0 shell interface
	RegCreateKeyEx(HKEY_CLASSES_ROOT,
		"urp_v1.0\\DefaultIcon",0, 
		NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,
		NULL, &keyHandle,&disposition);

	// set default icon for "urp_v1.0" shell interface.
	RegSetValueEx( keyHandle, NULL, 0, REG_SZ,
		(LPBYTE)lpszPathBuffer, lstrlen(lpszPathBuffer)+ 1 );

	// close registry key
	RegCloseKey(keyHandle);
	//.......>

	// create .urp file extension connection.
	RegCreateKeyEx(HKEY_CLASSES_ROOT,
		".urp",0, 
		NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,
		NULL, &keyHandle,&disposition);

	// connect .urp with "urp_v1.0" shell interface.
	RegSetValueEx( keyHandle, NULL, 0, REG_SZ,
		(LPBYTE)"urp_v1.0", 9 );

	// close registry key
	RegCloseKey(keyHandle);
	//.......>

	// file association is created notify the system.
	SHChangeNotify(SHCNE_ASSOCCHANGED,SHCNF_IDLIST,NULL,NULL);

}

// check and install/copy AlKatib1.TTF to fonts folder.
// if font installed/copy by this function then return value will be false.
BOOL Check_Install_Font(void)
{
	char lpBuffer[MAX_PATH];

	GetWindowsDirectory(lpBuffer,MAX_PATH);//  get the windows folder path to buffer

	PathAppend(lpBuffer,"Fonts\\AlKatib1.TTF");// append buffer to full Urdu font path for example C:\Windows to C:\Windows\Fonts\AlKatib1.TTF

	// if AlKatib1.TTF existed in fonts folder simply return true and dont install Urdu font.
	if (PathFileExists(lpBuffer) == TRUE)
		return TRUE;// AlKatib1 font is already installed so return.

	// /.......
	//Install/copy font to fonts folder from resource.

	// try to create new possibly C:\Windows\Fonts\AlKatib1.TTF
	HANDLE hfFont = CreateFile(lpBuffer,
		GENERIC_WRITE,FILE_SHARE_READ,NULL,
		CREATE_NEW,FILE_ATTRIBUTE_NORMAL,
		NULL);

	// check if new file is created
	if ( hfFont  == INVALID_HANDLE_VALUE )
		return TRUE;// error while creating new file so simply return.

	DWORD dwWrite;

	// find AlKatib1.TTF from resources
	HRSRC hRes = FindResource(NULL,"#108","URDU_FONT") ;

	DWORD dwResSize = SizeofResource(GetModuleHandle(NULL),hRes);// get the size of font resource data
	LPVOID lpResData = LockResource(LoadResource(NULL,hRes));// load and lock font resource data

	// write out whole font resource data to file
	WriteFile(hfFont,lpResData,dwResSize,&dwWrite,0);

	//FreeResource(???);

	// close file
	CloseHandle(hfFont);

	if (PathFileExists(lpBuffer) == TRUE)
	{
		// notify system for font change (not good way but ???) 
		SendMessage(HWND_BROADCAST,WM_FONTCHANGE,0,0);

		// return false to identify that we installed/copy font
		// with this action WinMain entry point function will rerun this program
		return FALSE;
	}


	return TRUE;
}

// size a child window to full parent client area.
void Size2ParentClientArea(HWND hWndChild,HWND hWndParent)
{
	RECT rect;

	// get parent window client area
	GetClientRect(hWndParent,&rect);

	// set position new values for child window
	SetWindowPos(hWndChild,NULL,0,0,rect.right,rect.bottom,SWP_NOZORDER);
}

// Fill Urdu Character Map
void InitCharMap(void)
{
	// fill whole phonetic(Urdu) character map with zeros later modify required values 
	ZeroMemory( &g_cmChar , sizeof(CharMap) * 266 );

	//--------------
	// map phonetic keyboard against Urdu character 
	//--------------

	g_cmChar['I'].iType = 1;
	g_cmChar['I'].char1 = TwoLaam_Hay_ful;

	g_cmChar['a'].iType = 2;
	g_cmChar['a'].char1 = Alif_ful;
	g_cmChar['a'].char4	= Alif_end;

	g_cmChar['A'].iType = 2;
	g_cmChar['A'].char1 = AlifMad_ful;
	g_cmChar['A'].char4= AlifMad_end;

	g_cmChar['b'].iType = 3;
	g_cmChar['b'].char1 = Bay_ful;
	g_cmChar['b'].char2 = Bay_sta;
	g_cmChar['b'].char3 = Bay_mid;
	g_cmChar['b'].char4 = Bay_end;

	g_cmChar['t'].iType = 3;
	g_cmChar['t'].char1 = Tay_ful;
	g_cmChar['t'].char2 = Tay_sta;
	g_cmChar['t'].char3 = Tay_mid;
	g_cmChar['t'].char4 = Tay_end;

	g_cmChar['C'].iType = 3;
	g_cmChar['C'].char1 = Say_ful;
	g_cmChar['C'].char2 = Say_sta;
	g_cmChar['C'].char3 = Say_mid;
	g_cmChar['C'].char4 = Say_end;

	g_cmChar['j'].iType = 3;
	g_cmChar['j'].char1 = Jeem_ful;
	g_cmChar['j'].char2 = Jeem_sta;
	g_cmChar['j'].char3 = Jeem_mid;
	g_cmChar['j'].char4 = Jeem_end;

	g_cmChar['H'].iType = 3;
	g_cmChar['H'].char1 = HayHuqqa_ful;
	g_cmChar['H'].char2 = HayHuqqa_sta;
	g_cmChar['H'].char3 = HayHuqqa_mid;
	g_cmChar['H'].char4 = HayHuqqa_end;


	g_cmChar['K'].iType = 3;
	g_cmChar['K'].char1 = Khay_ful;
	g_cmChar['K'].char2 = Khay_sta;
	g_cmChar['K'].char3 = Khay_mid;
	g_cmChar['K'].char4 = Khay_end;

	g_cmChar['d'].iType = 2;
	g_cmChar['d'].char1 = Daal_ful;
	g_cmChar['d'].char4	= Daal_end;

	g_cmChar['Z'].iType = 2;
	g_cmChar['Z'].char1 = Zaal_ful;
	g_cmChar['Z'].char4	= Zaal_end;

	g_cmChar['r'].iType = 2;
	g_cmChar['r'].char1 = Ray_ful;
	g_cmChar['r'].char4= Ray_end;

	g_cmChar['z'].iType = 2;
	g_cmChar['z'].char1 = Zay_ful;
	g_cmChar['z'].char4= Zay_end;

	g_cmChar['s'].iType = 3;
	g_cmChar['s'].char1 = Seen_ful;
	g_cmChar['s'].char2 = Seen_sta;
	g_cmChar['s'].char3 = Seen_mid;
	g_cmChar['s'].char4 = Seen_end;

	g_cmChar['x'].iType = 3;
	g_cmChar['x'].char1 = Sheen_ful;
	g_cmChar['x'].char2 = Sheen_sta;
	g_cmChar['x'].char3 = Sheen_mid;
	g_cmChar['x'].char4 = Sheen_end;

	g_cmChar['S'].iType = 3;
	g_cmChar['S'].char1 = Suad_ful;
	g_cmChar['S'].char2 = Suad_sta;
	g_cmChar['S'].char3 = Suad_mid;
	g_cmChar['S'].char4 = Suad_end;

	g_cmChar['J'].iType = 3;
	g_cmChar['J'].char1 = Duad_ful;
	g_cmChar['J'].char2 = Duad_sta;
	g_cmChar['J'].char3 = Duad_mid;
	g_cmChar['J'].char4 = Duad_end;

	g_cmChar['v'].iType = 3;
	g_cmChar['v'].char1 = Twain_ful;
	g_cmChar['v'].char2 = Twain_sta;
	g_cmChar['v'].char3 = Twain_mid;
	g_cmChar['v'].char4 = Twain_end;

	g_cmChar['p'].iType = 3;
	g_cmChar['p'].char1 = Pay_ful;
	g_cmChar['p'].char2 = Pay_sta;
	g_cmChar['p'].char3 = Pay_mid;
	g_cmChar['p'].char4 = Pay_end;

	g_cmChar['c'].iType = 3;
	g_cmChar['c'].char1 = Chay_ful;
	g_cmChar['c'].char2 = Chay_sta;
	g_cmChar['c'].char3 = Chay_mid;
	g_cmChar['c'].char4 = Chay_end;

	g_cmChar['X'].iType = 2;
	g_cmChar['X'].char1 = ZayZaala_ful;
	g_cmChar['X'].char4= ZayZaala_end;

	g_cmChar['g'].iType = 3;
	g_cmChar['g'].char1 = Gaaf_ful;
	g_cmChar['g'].char2 = Gaaf_sta;
	g_cmChar['g'].char3 = Gaaf_mid;
	g_cmChar['g'].char4 = Gaaf_end;

	g_cmChar['T'].iType = 3;
	g_cmChar['T'].char1 = TTay_ful;
	g_cmChar['T'].char2 = TTay_sta;
	g_cmChar['T'].char3 = TTay_mid;
	g_cmChar['T'].char4 = TTay_end;

	g_cmChar['D'].iType = 2;
	g_cmChar['D'].char1 = DDaal_ful;
	g_cmChar['D'].char4	= DDaal_end;

	g_cmChar['R'].iType = 2;
	g_cmChar['R'].char1 = RDay_ful;
	g_cmChar['R'].char4	= RDay_end;

	g_cmChar['V'].iType = 3;
	g_cmChar['V'].char1 = Zwain_ful;
	g_cmChar['V'].char2 = Zwain_sta;
	g_cmChar['V'].char3 = Zwain_mid;
	g_cmChar['V'].char4 = Zwain_end;

	g_cmChar['e'].iType = 3;
	g_cmChar['e'].char1 = Eain_ful;
	g_cmChar['e'].char2 = Eain_sta;
	g_cmChar['e'].char3 = Eain_mid;
	g_cmChar['e'].char4 = Eain_end;

	g_cmChar['G'].iType = 3;
	g_cmChar['G'].char1 = Ghain_ful;
	g_cmChar['G'].char2 = Ghain_sta;
	g_cmChar['G'].char3 = Ghain_mid;
	g_cmChar['G'].char4 = Ghain_end;

	g_cmChar['f'].iType = 3;
	g_cmChar['f'].char1 = Fay_ful;
	g_cmChar['f'].char2 = Fay_sta;
	g_cmChar['f'].char3 = Fay_mid;
	g_cmChar['f'].char4 = Fay_end;

	g_cmChar['q'].iType = 3;
	g_cmChar['q'].char1 = KKaaf_ful;
	g_cmChar['q'].char2 = KKaaf_sta;
	g_cmChar['q'].char3 = KKaaf_mid;
	g_cmChar['q'].char4 = KKaaf_end;

	g_cmChar['k'].iType = 3;
	g_cmChar['k'].char1 = Kaaf_ful;
	g_cmChar['k'].char2 = Kaaf_sta;
	g_cmChar['k'].char3 = Kaaf_mid;
	g_cmChar['k'].char4 = Kaaf_end;

	g_cmChar['l'].iType = 3;
	g_cmChar['l'].char1 = Laam_ful;
	g_cmChar['l'].char2 = Laam_sta;
	g_cmChar['l'].char3 = Laam_mid;
	g_cmChar['l'].char4 = Laam_end;

	g_cmChar['m'].iType = 3;
	g_cmChar['m'].char1 = Meem_ful;
	g_cmChar['m'].char2 = Meem_sta;
	g_cmChar['m'].char3 = Meem_mid;
	g_cmChar['m'].char4 = Meem_end;

	g_cmChar['n'].iType = 3;
	g_cmChar['n'].char1 = Noon_ful;
	g_cmChar['n'].char2 = Noon_sta;
	g_cmChar['n'].char3 = Noon_mid;
	g_cmChar['n'].char4 = Noon_end;

	g_cmChar['h'].iType = 3;
	g_cmChar['h'].char1 = HayTwoHeaded_ful;
	g_cmChar['h'].char2 = HayTwoHeaded_sta;
	g_cmChar['h'].char3 = HayTwoHeaded_mid;
	g_cmChar['h'].char4 = HayTwoHeaded_end;

	g_cmChar['w'].iType = 2;
	g_cmChar['w'].char1 = Wow_ful;
	g_cmChar['w'].char4	= Wow_end;

	g_cmChar['i'].iType = 3;
	g_cmChar['i'].char1 = YaySmall_ful;
	g_cmChar['i'].char2 = YaySmall_sta;
	g_cmChar['i'].char3 = YaySmall_mid;
	g_cmChar['i'].char4 = YaySmall_end;

	g_cmChar['N'].iType = 3;
	g_cmChar['N'].char1 = Noon_Ghunna_ful;
	g_cmChar['N'].char2 = Noon_Ghunna_sta;
	g_cmChar['N'].char3 = Noon_Ghunna_mid;
	g_cmChar['N'].char4 = Noon_Ghunna_end;

	g_cmChar['o'].iType = 3;
	g_cmChar['o'].char1 = Hay_ful;
	g_cmChar['o'].char2 = Hay_sta;
	g_cmChar['o'].char3 = Hay_mid;
	g_cmChar['o'].char4 = Hay_end;

	g_cmChar['y'].iType = 3;
	g_cmChar['y'].char1 = YayBig_ful;
	g_cmChar['y'].char2 = YayBig_sta;
	g_cmChar['y'].char3 = YayBig_mid;
	g_cmChar['y'].char4 = YayBig_end;

	g_cmChar['u'].iType = 3;
	g_cmChar['u'].char1 = Hamza_ful;
	g_cmChar['u'].char2 = Hamza_sta;
	g_cmChar['u'].char3 = Hamza_mid;
	g_cmChar['u'].char4 = Hamza_end;

	g_cmChar['L'].iType = 2;
	g_cmChar['L'].char1 = LaamAlif_ful;
	g_cmChar['L'].char4 = LaamAlif_end;

	g_cmChar['W'].iType = 2;
	g_cmChar['W'].char1 = WowHamza_ful;
	g_cmChar['W'].char4 = WowHamza_end;

	g_cmChar['P'].iType = 4;
	g_cmChar['P'].char5 = Pash_mix;

	g_cmChar['O'].iType = 4;
	g_cmChar['O'].char5 = Zuber_mix;

	g_cmChar['M'].iType = 4;
	g_cmChar['M'].char5 = Zeer_mix;

}

// Convert/Encode Phonetic Text into Urdu Text.
void Phonetic2Urdu(void)
{
	char szPhoText[TEXT_LIMIT],szUrText[TEXT_LIMIT];

	// get phonetic text from phonetic text box
	GetWindowText(g_hwndPhonetic,szPhoText,TEXT_LIMIT);
	lstrcpy(szUrText,szPhoText);// copy phonetic text to Urdu text buffer

	//,,,
	// example string and its map like this if phonetic text string = "Pakistan"  and  i = 3 then cn0 = i ,cn1 = s ,cp0 = a ,cp1 = P , szPhoText[i]= k
	//,,,
	int nLen = lstrlen(szPhoText);

	int i,j;
	unsigned char cn0,cn1,cp0,cp1;
	j = nLen ;
	for ( i = 0 ; i < nLen ; i++)
	{
		j--;
		cn0 = cn1 = cp0 = cp1 = 0 ;

		if ( i - 2 >= 0 )
			cp1 = szPhoText[ i - 2];
		if ( i - 1 >= 0 )
			cp0 = szPhoText[ i - 1];

		if ( i + 2 < nLen)
			cn1 = szPhoText[ i + 2];
		if ( i + 1 < nLen)
			cn0 = szPhoText[ i + 1];

		szUrText[j] = MapChar(cp0,cp1,szPhoText[i],cn0,cn1);
	}

	// set Urdu text to Urdu text box
	SetWindowText(g_hwndUrdu,szUrText);	
}

// Map a single phonetic character into Urdu character. 
char MapChar (char pre0,char pre1,char char0 ,char next0,char next1)
{
	unsigned char cPre,cNxt;
	int type = g_cmChar[char0].iType;

	if ( type == 0)
		return char0;

	if (type == 1 )
		return ( g_cmChar[char0].char1 ) ;

	if (type == 4)
		return ( g_cmChar[char0].char5 );

	if (g_cmChar[pre0].iType == 4)
		cPre = pre1;
	else 
		cPre = pre0;

	if (g_cmChar[next0].iType == 4)
		cNxt = next1;
	else
		cNxt = next0;

	if (type == 2) 
	{
		if (g_cmChar[cPre].iType == 3)
			return (g_cmChar[char0].char4);
		else
			return (g_cmChar[char0].char1);
	}

	if (type == 3) 
	{
		if (g_cmChar[cPre].iType == 0 || g_cmChar[cPre].iType == 1 ||g_cmChar[cPre].iType == 2 || g_cmChar[cPre].iType == 4)
		{
			if (g_cmChar[cNxt].iType == 0 || g_cmChar[cNxt].iType == 1 || g_cmChar[cNxt].iType == 4)
				return (g_cmChar[char0].char1);
			if (g_cmChar[cNxt].iType == 2 || g_cmChar[cNxt].iType == 3)
				return (g_cmChar[char0].char2);
		}
		if (g_cmChar[cPre].iType == 3)
		{
			if (g_cmChar[cNxt].iType == 2 || g_cmChar[cNxt].iType == 3)
				return (g_cmChar[char0].char3);
			if (g_cmChar[cNxt].iType == 0 || g_cmChar[cNxt].iType == 1 || g_cmChar[cNxt].iType == 4)
				return (g_cmChar[char0].char4);
		}
	}


	return(char0);
}
/*
This software created by Muhammad Arshad Latti
From Sargodha ( Pakistan ) 
e-mail   :: arshadlatti@gmail.com 
web_site :: http://www.geocities.com/arshadlati
*/