#include <Windows.h> 
#include "resource.h" 
#include <shlobj.h>
#include <shlwapi.h>
#include <commctrl.h>

#pragma comment( lib, "comctl32.lib" )
#pragma comment( lib, "shlwapi.lib" )

LRESULT CALLBACK StartupDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK AboutDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT APIENTRY _UrduSubProc(HWND hwnd,UINT uMsg, WPARAM wParam, LPARAM lParam);
char MapChar(char pre0,char pre1,char char0 ,char next0,char next1);
HFONT LoadFont_AlKatib1(void);
void Size2ParentClientArea(HWND hWndChild,HWND hWndParent);
void GetFileName_Open(LPSTR lpszOpenFile , HWND hWndParent);
void GetFileName_Save(LPSTR lpszSaveFile , HWND hWndParent);
void SaveUrdu2File(void);
void LoadUrduFromFile(LPSTR lpUrduFile);
void SetFileAssociation(void);
void OpenUrduFile(void);
void InitCharMap(void);
void Phonetic2Urdu(void);
void CmdLoadFile(void);
BOOL Check_Install_Font(void);

#define TEXT_LIMIT		10240 // 10 KB text limit
#define URP_HEADER	"UPTW_ARSHAD_1.0" //.urp file header
#define URP_HEADER_SIZE	15 //.urp file header size		

// Phonetic (Urdu) character map as in my algorithm for writing Urdu script.
struct CharMap
{
	int  iType;
	//unsigned char char0;//	Not used at movement 
	unsigned char char1;
	unsigned char char2;
	unsigned char char3;
	unsigned char char4;
	unsigned char char5;
};

//AlKatib1 Urdu font map
enum AlKatib1_Map 
{
	TwoLaam_Hay_ful	= 34,
	Alif_ful		= 65,
	Alif_end        = 66,
	AlifMad_ful		= 69,
	AlifMad_end		= 70,
	Bay_sta			= 73,
	Bay_mid			= 74,
	Bay_end			= 75,
	Bay_ful			= 76,
	Tay_sta			= 77,
	Tay_mid			= 78,
	Tay_end			= 79,
	Tay_ful			= 80,
	Say_sta			= 81,
	Say_mid			= 82,
	Say_end			= 83,
	Say_ful			= 84,
	Jeem_sta		= 85,
	Jeem_mid		= 86,
	Jeem_end		= 87,
	Jeem_ful		= 88,
	HayHuqqa_sta	= 89,
	HayHuqqa_mid	= 90,
	HayHuqqa_end	= 92,
	HayHuqqa_ful	= 96,
	Khay_sta		= 97,
	Khay_mid		= 98,
	Khay_end		= 99,
	Khay_ful		=100,
	Daal_ful		=101,
	Daal_end		=102,
	Zaal_ful		=103,
	Zaal_end		=104,
	Ray_ful			=105,
	Ray_end			=106,
	Zay_ful			=107,
	Zay_end			=108,
	Seen_sta		=109,
	Seen_mid		=110,
	Seen_end		=111,
	Seen_ful		=112,
	Sheen_sta		=113,
	Sheen_mid		=114,
	Sheen_end		=115,
	Sheen_ful		=116,
	Suad_sta		=117,
	Suad_mid		=118,
	Suad_end		=119,
	Suad_ful		=120,
	Duad_sta		=121,
	Duad_mid		=122,
	Duad_end		=124,
	Duad_ful		=126,
	Twain_sta		=162,
	Twain_mid		=241,
	Twain_end		=161,
	Twain_ful		=127,
	Pay_sta			=130,
	Pay_mid			=131,
	Pay_end			=132,
	Pay_ful			=133,
	Chay_sta		=134,
	Chay_mid		=135,
	Chay_end		=136,
	Chay_ful		=137,
	ZayZaala_ful	=138,
	ZayZaala_end	=139,
	Gaaf_sta		=140,
	Gaaf_mid		=141,
	Gaaf_end		=143,//????
	Gaaf_ful		=143,
	TTay_sta		=144,
	TTay_mid		=145,
	TTay_end		=146,
	TTay_ful		=147,
	DDaal_ful		=148,
	DDaal_end		=149,
	RDay_ful		=152,
	RDay_end		=153,
	Zwain_sta		=166,
	Zwain_mid		=165,
	Zwain_end		=164,
	Zwain_ful		=163,
	Eain_sta		=167,
	Eain_mid		=168,
	Eain_end		=169,
	Eain_ful		=170,
	Ghain_sta		=171,
	Ghain_mid		=172,
	Ghain_end		=173,
	Ghain_ful		=174,
	Fay_sta			=175,
	Fay_mid			=176,
	Fay_end			=177,
	Fay_ful			=178,
	KKaaf_sta		=179,
	KKaaf_mid		=180,
	KKaaf_end		=181,
	KKaaf_ful		=182,
	Kaaf_sta		=183,
	Kaaf_mid		=184,
	Kaaf_end		=185,
	Kaaf_ful		=186,
	Laam_sta		=187,
	Laam_mid		=188,
	Laam_end		=189,
	Laam_ful		=190,
	Meem_sta		=191,
	Meem_mid		=192,
	Meem_end		=193,
	Meem_ful		=194,
	Noon_sta		=195,
	Noon_mid		=196,
	Noon_end		=197,
	Noon_ful		=198,
	HayHeaded_sta	=199,//........
	HayHeaded_mid	=200,
	HayHeaded_end	=201,
	HayHeaded_ful	=202,//........
	Wow_ful			=203,
	Wow_end			=204,
	YaySmall_sta	=205,
	YaySmall_mid	=206,
	YaySmall_end	=211,
	YaySmall_ful	=212,
	Noon_Ghunna_sta	=228,
	Noon_Ghunna_mid	=229,
	Noon_Ghunna_end	=230,
	Noon_Ghunna_ful	=231,
	HayTwoHeaded_sta=232,
	HayTwoHeaded_mid=233,
	HayTwoHeaded_end=234,
	HayTwoHeaded_ful=235,
	Hay_sta			=236,
	Hay_mid			=237,
	Hay_end			=238,
	Hay_ful			=202,
	YayBig_sta		=205,
	YayBig_mid		=206,
	YayBig_end		=154,
	YayBig_ful		=155,
	Hamza_sta		=214,
	Hamza_mid		=215,
	Hamza_end		=239,
	Hamza_ful		=213,
	LaamAlif_ful	=220,
	LaamAlif_end	=221,
	WowHamza_ful	=218,
	WowHamza_end	=219,
	HamzaOnly_ful	=213,
	Pash_mix		=243,
	Zuber_mix		=242,
	Zeer_mix		=248
};
/*
This software created by Muhammad Arshad Latti
From Sargodha ( Pakistan ) 
e-mail   :: arshadlatti@gmail.com 
web_site :: http://www.geocities.com/arshadlati
*/